@@ -35,28 +35,31 @@
 #include "prefs.h"
 #include "gfx.h"
 #include "games-gridframe.h"
-#include "games-fullscreen-action.h"
 #include "games-stock.h"
 
 #define SPEED_MOVE     25
 #define SPEED_DROP     20
 #define SPEED_BLINK    150
 
-#define DEFAULT_WIDTH 350
-#define DEFAULT_HEIGHT 390
+#define DEFAULT_WIDTH 495
+#define DEFAULT_HEIGHT 435
 
 extern Prefs p;
 
 GSettings *settings;
-GtkWidget *app;
-GtkWidget *notebook;
+GtkWidget *window;
 GtkWidget *drawarea;
-GtkWidget *statusbar;
+GtkWidget *headerbar;
 GtkWidget *scorebox = NULL;
+static GtkApplication *application;
 
 GtkWidget *label_name[3];
 GtkWidget *label_score[3];
 
+GAction *new_game_action;
+GAction *undo_action;
+GAction *hint_action;
+
 PlayerID player;
 PlayerID winner;
 PlayerID who_starts;
@@ -349,21 +352,10 @@ swap_player (void)
 void
 set_status_message (const gchar * message)
 {
-  guint context_id;
-  context_id =
-    gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "message");
-  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
   if (message)
-    gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, message);
+    gtk_header_bar_set_title (GTK_HEADER_BAR (headerbar), message);
 }
 
-
-GtkAction *new_game_action;
-GtkAction *undo_action;
-GtkAction *hint_action;
-GtkAction *fullscreen_action;
-
-
 static void
 stop_anim (void)
 {
@@ -400,8 +392,9 @@ void
 game_reset (void)
 {
   stop_anim ();
-  gtk_action_set_sensitive (undo_action, FALSE);
-  gtk_action_set_sensitive (hint_action, FALSE);
+
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), FALSE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (hint_action), FALSE);
 
   who_starts = (who_starts == PLAYER1) ? PLAYER2 : PLAYER1;
   player = who_starts;
@@ -496,31 +489,26 @@ prompt_player (void)
   const gchar *who = NULL;
   gchar *str = NULL;
 
-  gtk_action_set_visible (new_game_action, TRUE);
-  gtk_action_set_visible (hint_action, TRUE);
-  gtk_action_set_visible (undo_action, TRUE);
-
-  gtk_action_set_sensitive (new_game_action, (human || gameover));
-  gtk_action_set_sensitive (hint_action, (human || gameover));
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (hint_action), (human || gameover));
 
   switch (players) {
   case 0:
-    gtk_action_set_sensitive (undo_action, FALSE);
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), FALSE);
     break;
   case 1:
-    gtk_action_set_sensitive (undo_action,
-			      ((human && moves > 1) || (!human && gameover)));
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action),
+                              ((human && moves > 1) || (!human && gameover)));
     break;
   case 2:
-    gtk_action_set_sensitive (undo_action, (moves > 0));
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), (moves > 0));
     break;
   }
 
   if (gameover && winner == NOBODY) {
     if (score[NOBODY] == 0) {
-      set_status_message ("");
+      set_status_message (NULL);
     } else {
-      set_status_message (_("It's a draw!"));
+      set_status_message (_("It’s a draw!"));
     }
     return;
   }
@@ -531,12 +519,12 @@ prompt_player (void)
       if (gameover)
 	set_status_message (_("You win!"));
       else
-	set_status_message (_("It is your move."));
+	set_status_message (_("Your Turn"));
     } else {
       if (gameover)
 	set_status_message (_("I win!"));
       else
-	set_status_message (_("Thinking..."));
+	set_status_message (_("I’m Thinking…"));
     }
     break;
   case 2:
@@ -549,12 +537,12 @@ prompt_player (void)
     if (gameover) {
 	str = g_strdup_printf (_("%s wins!"), who);
       }
-    if (player_active) {
-      set_status_message (_("It is your move."));
+    else if (player_active) {
+      set_status_message (_("Your Turn"));
       return;
 
     } else {
-      str = g_strdup_printf (_("Waiting for %s to move."), who);
+      str = g_strdup_printf (_("%s’s Turn"), who);
     }
 
     set_status_message (str);
@@ -566,23 +554,22 @@ prompt_player (void)
 
 
 static void
-on_game_new (void)
+on_game_new (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   stop_anim ();
   game_reset ();
 }
 
-static gboolean
-on_game_exit (GObject * object, gpointer data)
+static void
+on_game_exit (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
 
   stop_anim ();
-  gtk_main_quit ();
-  return TRUE;
+  g_application_quit (G_APPLICATION (application));
 }
 
 static void
-on_game_undo (GtkMenuItem * m, gpointer data)
+on_game_undo (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   gint r, c;
 
@@ -625,7 +612,7 @@ on_game_undo (GtkMenuItem * m, gpointer data)
 
 
 static void
-on_game_hint (GtkMenuItem * m, gpointer data)
+on_game_hint (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   gchar *s;
   gint c;
@@ -635,23 +622,19 @@ on_game_hint (GtkMenuItem * m, gpointer data)
   if (gameover)
     return;
 
-  gtk_action_set_sensitive (hint_action, FALSE);
-  gtk_action_set_sensitive (undo_action, FALSE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (hint_action), FALSE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), FALSE);
 
-  set_status_message (_("Thinking..."));
+  set_status_message (_("I’m Thinking…"));
 
   vstr[0] = vlevel[LEVEL_STRONG];
   c = playgame (vstr, vboard) - 1;
 
   column_moveto = c;
-  if (p.do_animate) {
-    while (timeout)
-      gtk_main_iteration ();
-    anim = ANIM_HINT;
-    timeout = g_timeout_add (SPEED_MOVE, (GSourceFunc) on_animate, NULL);
-  } else {
-    move_cursor (column_moveto);
-  }
+  while (timeout)
+    gtk_main_iteration ();
+  anim = ANIM_HINT;
+  timeout = g_timeout_add (SPEED_MOVE, (GSourceFunc) on_animate, NULL);
 
   blink_tile (0, c, gboard[0][c], 6);
 
@@ -659,12 +642,9 @@ on_game_hint (GtkMenuItem * m, gpointer data)
   set_status_message (s);
   g_free (s);
 
-  gtk_action_set_sensitive (hint_action, TRUE);
-  gtk_action_set_sensitive (undo_action, (moves > 0));
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), (moves > 0));
 }
 
-
-
 void
 on_dialog_close (GtkWidget * w, int response_id, gpointer data)
 {
@@ -723,7 +703,7 @@ scorebox_reset (void)
 
 
 static void
-on_game_scores (GtkMenuItem * m, gpointer data)
+on_game_scores (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   GtkWidget *grid, *grid2, *icon;
 
@@ -733,9 +713,9 @@ on_game_scores (GtkMenuItem * m, gpointer data)
   }
 
   scorebox = gtk_dialog_new_with_buttons (_("Scores"),
-					  GTK_WINDOW (app),
+					  GTK_WINDOW (window),
 					  GTK_DIALOG_DESTROY_WITH_PARENT,
-					  GTK_STOCK_CLOSE,
+					  _("_Close"),
 					  GTK_RESPONSE_CLOSE, NULL);
 
   gtk_window_set_resizable (GTK_WINDOW (scorebox), FALSE);
@@ -753,9 +733,6 @@ on_game_scores (GtkMenuItem * m, gpointer data)
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (scorebox))),
 		      grid, TRUE, TRUE, 0);
 
-  icon = gtk_image_new_from_icon_name ("four-in-a-row", GTK_ICON_SIZE_DIALOG);
-  gtk_container_add (GTK_CONTAINER (grid), icon);
-
   grid2 = gtk_grid_new ();
   gtk_container_add (GTK_CONTAINER (grid), grid2);
   gtk_grid_set_column_spacing (GTK_GRID (grid2), 6);
@@ -792,10 +769,8 @@ on_game_scores (GtkMenuItem * m, gpointer data)
   scorebox_update ();
 }
 
-
-
 static void
-on_help_about (GtkAction * action, gpointer data)
+on_help_about (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   const gchar *authors[] = { "Four-in-a-row:",
     "  Tim Musson <trmusson@ihug.co.nz>",
@@ -811,6 +786,8 @@ on_help_about (GtkAction * action, gpointer data)
 
   const gchar *artists[] = { "Alan Horkan",
     "Tim Musson",
+    "Anatol Drlicek",
+    "Based on the Faenza icon theme by Matthieu James",
     NULL
   };
 
@@ -818,29 +795,28 @@ on_help_about (GtkAction * action, gpointer data)
     NULL
   };
 
-  gtk_show_about_dialog (GTK_WINDOW (app),
+  gtk_show_about_dialog (GTK_WINDOW (window),
 			 "name", _(APPNAME_LONG),
 			 "version", VERSION,
 			 "copyright",
-			 "Copyright \xc2\xa9 1999-2008, Tim Musson and David Neary",
+			 "Copyright © 1999–2008, Tim Musson and David Neary",
 			 "license-type", GTK_LICENSE_GPL_2_0, "comments",
 		         _("Connect four in a row to win.\n\nFour-in-a-row is a part of GNOME Games."),
-		         "website-label", _("GNOME Games web site"),
 			 "authors", authors, "documenters", documenters,
 			 "artists", artists, "translator-credits",
 			 _("translator-credits"),
 			 "logo-icon-name", "four-in-a-row",
-			 "website", "http://www.gnome.org/projects/gnome-games/",
+			 "website", "https://wiki.gnome.org/Apps/Four-in-a-row",
 			 NULL);
 }
 
 
 static void
-on_help_contents (GtkAction * action, gpointer data)
+on_help_contents (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
   GError *error = NULL;
 
-  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (app)), "help:four-in-a-row", gtk_get_current_event_time (), &error);
+  gtk_show_uri (gtk_widget_get_screen (window), "help:four-in-a-row", gtk_get_current_event_time (), &error);
   if (error)
     g_warning ("Failed to show help: %s", error->message);
   g_clear_error (&error);
@@ -848,7 +824,7 @@ on_help_contents (GtkAction * action, gpointer data)
 
 
 static void
-on_settings_preferences (GtkAction * action, gpointer user_data)
+on_settings_preferences (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
   prefsbox_open ();
 }
@@ -1048,16 +1024,10 @@ process_move (gint c)
 
   }
 
-  if (!p.do_animate) {
-    move_cursor (c);
-    column_moveto = c;
-    process_move2 (c);
-  } else {
-    column_moveto = c;
-    anim = ANIM_MOVE;
-    timeout = g_timeout_add (SPEED_MOVE,
-                             (GSourceFunc) on_animate, GINT_TO_POINTER (c));
-  }
+  column_moveto = c;
+  anim = ANIM_MOVE;
+  timeout = g_timeout_add (SPEED_MOVE,
+                           (GSourceFunc) on_animate, GINT_TO_POINTER (c));
 }
 
 static void
@@ -1067,18 +1037,12 @@ process_move2 (gint c)
 
   r = first_empty_row (c);
   if (r > 0) {
-
-    if (!p.do_animate) {
-      drop_marble (r, c);
-      process_move3 (c);
-    } else {
-      row = 0;
-      row_dropto = r;
-      anim = ANIM_DROP;
-      timeout = g_timeout_add (SPEED_DROP,
-                               (GSourceFunc) on_animate,
-                               GINT_TO_POINTER (c));
-    }
+    row = 0;
+    row_dropto = r;
+    anim = ANIM_DROP;
+    timeout = g_timeout_add (SPEED_DROP,
+                             (GSourceFunc) on_animate,
+                             GINT_TO_POINTER (c));
   } else {
     play_sound (SOUND_COLUMN_FULL);
   }
@@ -1176,133 +1140,76 @@ on_button_press (GtkWidget * w, GdkEventButton * e, gpointer data)
   return TRUE;
 }
 
-static const GtkActionEntry action_entry[] = {
-  {"GameMenu", NULL, N_("_Game")},
-  {"ViewMenu", NULL, N_("_View")},
-  {"SettingsMenu", NULL, N_("_Settings")},
-  {"HelpMenu", NULL, N_("_Help")},
-  {"NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL,
-   G_CALLBACK (on_game_new)},
-  {"UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, NULL,
-   G_CALLBACK (on_game_undo)},
-  {"Hint", GAMES_STOCK_HINT, NULL, NULL, NULL, G_CALLBACK (on_game_hint)},
-  {"Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL,
-   G_CALLBACK (on_game_scores)},
-  {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (on_game_exit)},
-  {"Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
-   G_CALLBACK (on_settings_preferences)},
-  {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL,
-   G_CALLBACK (on_help_contents)},
-  {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (on_help_about)}
+static const GActionEntry app_entries[] = {
+  {"new-game", on_game_new, NULL, NULL, NULL},
+  {"undo-move", on_game_undo, NULL, NULL, NULL},
+  {"hint", on_game_hint, NULL, NULL, NULL},
+  {"scores", on_game_scores, NULL, NULL, NULL},
+  {"quit", on_game_exit, NULL, NULL, NULL},
+  {"preferences", on_settings_preferences, NULL, NULL, NULL},
+  {"help", on_help_contents, NULL, NULL, NULL},
+  {"about", on_help_about, NULL, NULL, NULL}
 };
 
-static const char ui_description[] =
-  "<ui>"
-  "  <menubar name='MainMenu'>"
-  "    <menu action='GameMenu'>"
-  "      <menuitem action='NewGame'/>"
-  "      <separator/>"
-  "      <menuitem action='UndoMove'/>"
-  "      <menuitem action='Hint'/>"
-  "      <separator/>"
-  "      <menuitem action='Scores'/>"
-  "      <separator/>"
-  "      <menuitem action='Quit'/>"
-  "    </menu>"
-  "    <menu action='ViewMenu'>"
-  "      <menuitem action='Fullscreen'/>"
-  "    </menu>"
-  "    <menu action='SettingsMenu'>"
-  "      <menuitem action='Preferences'/>"
-  "    </menu>"
-  "    <menu action='HelpMenu'>"
-  "      <menuitem action='Contents'/>"
-  "      <menuitem action='About'/>" "    </menu>" "  </menubar>" "</ui>";
-
-
-
-static void
-create_game_menus (GtkUIManager * ui_manager)
-{
-  GtkActionGroup *action_group;
-  games_stock_init ();
-
-  action_group = gtk_action_group_new ("actions");
-
-  gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-  gtk_action_group_add_actions (action_group, action_entry,
-				G_N_ELEMENTS (action_entry), app);
-
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
-  gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
-
-  gtk_window_add_accel_group (GTK_WINDOW (app),
-			      gtk_ui_manager_get_accel_group (ui_manager));
-
-  new_game_action = gtk_action_group_get_action (action_group, "NewGame");
-
-  hint_action = gtk_action_group_get_action (action_group, "Hint");
-  undo_action = gtk_action_group_get_action (action_group, "UndoMove");
-  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW (app)));
-  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
-}
-
-
 static gboolean
 create_app (void)
 {
-  GtkWidget *menubar;
   GtkWidget *gridframe;
-  GtkWidget *grid;
-  GtkWidget *vpaned;
-  GtkUIManager *ui_manager;
-
-  app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (app), _(APPNAME_LONG));
-
-  gtk_window_set_default_size (GTK_WINDOW (app), DEFAULT_WIDTH, DEFAULT_HEIGHT);
-  //games_conf_add_window (GTK_WINDOW (app), NULL);
-
-  notebook = gtk_notebook_new ();
-  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
-  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
-
-  g_signal_connect (G_OBJECT (app), "delete_event",
-		    G_CALLBACK (on_game_exit), NULL);
+  GtkWidget *hbox, *vbox;
+  GtkWidget *image;
+  GtkWidget *new_game_button;
+  GtkWidget *undo_button;
+  GtkWidget *hint_button;
+  GMenu *app_menu, *section;
 
+  window = gtk_application_window_new (application);
+  gtk_window_set_application (GTK_WINDOW (window), application);
+  gtk_window_set_title (GTK_WINDOW (window), _(APPNAME_LONG));
+  gtk_window_set_default_size (GTK_WINDOW (window), DEFAULT_WIDTH, DEFAULT_HEIGHT);
   gtk_window_set_default_icon_name ("four-in-a-row");
 
-  statusbar = gtk_statusbar_new ();
-  ui_manager = gtk_ui_manager_new ();
+  headerbar = gtk_header_bar_new ();
+  gtk_header_bar_set_title (GTK_HEADER_BAR (headerbar), _("Four-in-a-row"));
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (headerbar), TRUE);
+  gtk_widget_show (headerbar);
+  gtk_window_set_titlebar (GTK_WINDOW (window), headerbar);
 
-  games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
-  create_game_menus (ui_manager);
-  menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+  g_action_map_add_action_entries (G_ACTION_MAP (application), app_entries, G_N_ELEMENTS (app_entries), application);
+  gtk_application_add_accelerator (application, "<Primary>n", "app.new-game", NULL);
+  gtk_application_add_accelerator (application, "<Primary>h", "app.hint", NULL);
+  gtk_application_add_accelerator (application, "<Primary>z", "app.undo-move", NULL);
+  gtk_application_add_accelerator (application, "<Primary>q", "app.quit", NULL);
+  gtk_application_add_accelerator (application, "F1", "app.contents", NULL);
 
-  vpaned = gtk_paned_new (GTK_ORIENTATION_VERTICAL);
-  gtk_widget_set_hexpand (vpaned, TRUE);
-  gtk_widget_set_vexpand (vpaned, TRUE);
+  app_menu = g_menu_new ();
+  section = g_menu_new ();
+  g_menu_append_section (app_menu, NULL, G_MENU_MODEL (section));
+  g_menu_append (section, _("_New Game"), "app.new-game");
+  g_menu_append (section, _("_Scores"), "app.scores");
+  g_menu_append (section, _("_Preferences"), "app.preferences");
+  section = g_menu_new ();
+  g_menu_append_section (app_menu, NULL, G_MENU_MODEL (section));
+  g_menu_append (section, _("_Help"), "app.help");
+  g_menu_append (section, _("_About"), "app.about");
+  g_menu_append (section, _("_Quit"), "app.quit");
 
-  grid = gtk_grid_new ();
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+  new_game_action = g_action_map_lookup_action (G_ACTION_MAP (application), "new-game");
+  undo_action = g_action_map_lookup_action (G_ACTION_MAP (application), "undo-move");
+  hint_action = g_action_map_lookup_action (G_ACTION_MAP (application), "hint");
+
+  gtk_application_set_app_menu (GTK_APPLICATION (application), G_MENU_MODEL (app_menu));
+
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_container_add (GTK_CONTAINER (window), hbox);
 
   gridframe = games_grid_frame_new (7, 7);
-
-  gtk_paned_pack1 (GTK_PANED (vpaned), gridframe, TRUE, FALSE);
-
-  gtk_container_add (GTK_CONTAINER (grid), menubar);
-  gtk_container_add (GTK_CONTAINER (grid), vpaned);
-  gtk_container_add (GTK_CONTAINER (grid), statusbar);
-
-  gtk_container_add (GTK_CONTAINER (app), notebook);
-  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), grid, NULL);
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), MAIN_PAGE);
+  gtk_widget_set_hexpand (gridframe, TRUE);
+  gtk_widget_set_vexpand (gridframe, TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), gridframe, TRUE, TRUE, 10);
 
   drawarea = gtk_drawing_area_new ();
-
   /* set a min size to avoid pathological behavior of gtk when scaling down */
-  gtk_widget_set_size_request (drawarea, 200, 200);
-
+  gtk_widget_set_size_request (drawarea, 300, 300);
   gtk_container_add (GTK_CONTAINER (gridframe), drawarea);
 
   gtk_widget_set_events (drawarea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
@@ -1312,16 +1219,53 @@ create_app (void)
 		    G_CALLBACK (on_drawarea_draw), NULL);
   g_signal_connect (G_OBJECT (drawarea), "button_press_event",
 		    G_CALLBACK (on_button_press), NULL);
-  g_signal_connect (G_OBJECT (app), "key_press_event",
+  g_signal_connect (G_OBJECT (window), "key_press_event",
 		    G_CALLBACK (on_key_press), NULL);
 
   /* We do our own double-buffering. */
   gtk_widget_set_double_buffered (GTK_WIDGET (drawarea), FALSE);
 
-  gtk_action_set_sensitive (hint_action, FALSE);
-  gtk_action_set_sensitive (undo_action, FALSE);
+  if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
+    image = gtk_image_new_from_icon_name ("edit-undo-rtl-symbolic", GTK_ICON_SIZE_DIALOG);
+  else
+    image = gtk_image_new_from_icon_name ("edit-undo-symbolic", GTK_ICON_SIZE_DIALOG);
 
-  gtk_widget_show_all (app);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 6);
+
+  undo_button = gtk_button_new ();
+  gtk_widget_set_valign (undo_button, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (undo_button, GTK_ALIGN_CENTER);
+  gtk_button_set_image (GTK_BUTTON (undo_button), image);
+  gtk_button_set_relief (GTK_BUTTON (undo_button), GTK_RELIEF_NONE);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (undo_button), "app.undo-move");
+  gtk_widget_set_tooltip_text (undo_button, _("Undo your most recent move"));
+  gtk_box_pack_start (GTK_BOX (vbox), undo_button, FALSE, FALSE, 0);
+
+  hint_button = gtk_button_new ();
+  gtk_widget_set_valign (hint_button, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (hint_button, GTK_ALIGN_CENTER);
+  image = gtk_image_new_from_icon_name ("dialog-question-symbolic", GTK_ICON_SIZE_DIALOG);
+  gtk_button_set_image (GTK_BUTTON (hint_button), image);
+  gtk_button_set_relief (GTK_BUTTON (hint_button), GTK_RELIEF_NONE);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (hint_button), "app.hint");
+  gtk_widget_set_tooltip_text (hint_button, _("Receive a hint for your next move"));
+  gtk_box_pack_start (GTK_BOX (vbox), hint_button, FALSE, FALSE, 0);
+
+  new_game_button = gtk_button_new ();
+  gtk_widget_set_valign (new_game_button, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (new_game_button, GTK_ALIGN_CENTER);
+  image = gtk_image_new_from_icon_name ("view-refresh-symbolic", GTK_ICON_SIZE_DIALOG);
+  gtk_button_set_image (GTK_BUTTON (new_game_button), image);
+  gtk_button_set_relief (GTK_BUTTON (new_game_button), GTK_RELIEF_NONE);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (new_game_button), "app.new-game");
+  gtk_widget_set_tooltip_text (new_game_button, _("Start a new game"));
+  gtk_box_pack_end (GTK_BOX (vbox), new_game_button, FALSE, FALSE, 0);
+
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (hint_action), FALSE);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (undo_action), FALSE);
+
+  gtk_widget_show_all (window);
 
   gfx_refresh_pixmaps ();
   gfx_draw_all ();
@@ -1329,6 +1273,7 @@ create_app (void)
   scorebox_update ();		/* update visible player descriptions */
   prompt_player ();
 
+  game_reset ();
   return TRUE;
 }
 
@@ -1340,17 +1285,15 @@ main (int argc, char *argv[])
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
+  gint app_retval;
 
   setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  /*
-   * Required because the binary doesn't match the desktop file.
-   * Has to be before the call to g_option_context_parse.
-   */
-  g_set_prgname (APPNAME);
+  application = gtk_application_new ("org.gnome.four-in-a-row", 0);
+  g_signal_connect (application, "activate", G_CALLBACK (create_app), NULL);
 
   context = g_option_context_new (NULL);
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
@@ -1374,12 +1317,9 @@ main (int argc, char *argv[])
     exit (1);
   }
 
-  if (create_app ()) {
-    game_reset ();
-    gtk_main ();
-  }
+  app_retval = g_application_run (G_APPLICATION (application), argc, argv);
 
   game_free ();
 
-  return 0;
+  return app_retval;
 }
